//-----------------------------------------------------------------------------
// File: mp_wrap.h
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#ifndef MAGIC_PARTICLES_DX
#define MAGIC_PARTICLES_DX

#include "mp.h"
#include <d3d9.h>
#include <d3dx9tex.h>
#pragma comment (lib, "d3d9.lib")
#pragma comment (lib, "d3dx9.lib")

#include "magic.h"

#define MAX_SEGMENT	4
#define SIZE_SEGMENT 512

#ifdef MAGIC_3D
#define FVF_PARTICLE (D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1)
#else
#define FVF_PARTICLE (D3DFVF_XYZRHW | D3DFVF_DIFFUSE | D3DFVF_TEX1)
#endif

struct VERTEX_PARTICLE
{
	float x, y, z;		// eng: coordinates
	// rus: 
#ifndef MAGIC_3D
	float rhw;
#endif

	unsigned long color;// eng: color
	// rus: 

	float u, v;			// eng: textural coordinates
	// rus:  
};

/*
//   
class DX_BUFFER
{
protected:
	void* locked_buffer;

	int bytes_per_one;	//    1  

	int max_length;		//     

public:
	DX_BUFFER();
	
	virtual ~DX_BUFFER(){}

	void* Lock(LPDIRECT3DDEVICE9 device_3d, MAGIC_ARRAY_INFO* info, int additional_vertex_position);
	virtual void Unlock(LPDIRECT3DDEVICE9 device_3d)=0;

protected:
	virtual void Create(LPDIRECT3DDEVICE9 device_3d, int new_length)=0;
	virtual void LockBuffer(int offset, int size, int additional_vertex_position)=0;
};

//   
class DX_VERTEX_BUFFER : public DX_BUFFER
{
protected:
	LPDIRECT3DVERTEXBUFFER9 vertex_buffer;

	int max_length_copy;
	int bytes_per_one_copy;

public:
	DX_VERTEX_BUFFER();
	virtual ~DX_VERTEX_BUFFER();

	virtual void Clear();

	virtual void Unlock(LPDIRECT3DDEVICE9 device_3d);

protected:
	virtual void Create(LPDIRECT3DDEVICE9 device_3d, int new_length);
	virtual void LockBuffer(int offset, int size, int additional_vertex_position);
};

//   
class DX_INDEX_BUFFER : public DX_BUFFER
{
protected:
	LPDIRECT3DINDEXBUFFER9 index_buffer;

public:
	DX_INDEX_BUFFER();
	virtual ~DX_INDEX_BUFFER();

	virtual void Clear();

	virtual void Unlock(LPDIRECT3DDEVICE9 device_3d);

protected:
	virtual void Create(LPDIRECT3DDEVICE9 device_3d, int new_length);
	void LockBuffer(int offset, int size, int);
};
*/

// eng: Class storing the textural atlas
// rus: ,    
class MP_Atlas_WRAP : public MP_Atlas
{
protected:
	LPDIRECT3DTEXTURE9 texture;

public:
	MP_Atlas_WRAP(int width, int height, const char* file);

	// eng: Destroy atlas texture
	// rus:   
	virtual void Destroy();

	// eng: Returns texture
	// rus:  
	LPDIRECT3DTEXTURE9 GetTexture(){return texture;}

	// eng: Loading of frame texture
	// rus:   
	virtual void LoadTexture(const MAGIC_CHANGE_ATLAS* c);

	// eng: Cleaning up of rectangle
	// rus:  
	virtual void CleanRectangle(const MAGIC_CHANGE_ATLAS* c);
};

struct MP_LAYOUT_WRAP : public MP_LAYOUT
{
	IDirect3DVertexDeclaration9* declaration;

	MP_LAYOUT_WRAP();
	virtual ~MP_LAYOUT_WRAP();

	virtual void Generate(MAGIC_VERTEX_FORMAT* format, MP_Device* device);
};

struct MP_VERTEX_BUFFER_WRAP : public MP_BUFFER
{
	LPDIRECT3DVERTEXBUFFER9 buffer;

	int filled_length, filled_stride;

	MP_VERTEX_BUFFER_WRAP();

	virtual void Create(int new_length, MP_Device* device);
	virtual void Destroy();
	virtual void* Map(int  stride, MP_Device* device);
	virtual void Unmap(MP_Device* device);
	virtual void Set(MAGIC_VERTEX_FORMAT* format, int stride, MP_Device* device);
};

struct MP_INDEX_BUFFER_WRAP : public MP_BUFFER
{
	LPDIRECT3DINDEXBUFFER9 buffer;

	MP_INDEX_BUFFER_WRAP();

	virtual void Create(int new_length, MP_Device* device);
	virtual void Destroy();
	virtual void* Map(int stride, MP_Device* device);
	virtual void Unmap(MP_Device* device);
	virtual void Set(MAGIC_VERTEX_FORMAT* format, int stride, MP_Device* device);
};


// eng: Class controlling drawing
// rus: ,   
class MP_Device_WRAP : public MP_Device
{
protected:
	HWND hwnd;

	LPDIRECT3D9 direct_3d;
	LPDIRECT3DDEVICE9 device_3d;

	DWORD state_addressu;
	DWORD state_addressv;
	DWORD state_magfilter;
	DWORD state_minfilter;
	DWORD state_colorarg1;
	DWORD state_colorarg2;
	DWORD state_colorop;
	DWORD state_alphaarg1;
	DWORD state_alphaarg2;
	DWORD state_alphaop;
	DWORD state_srcblend;
	DWORD state_alphaenable;
	DWORD state_lighting;
	DWORD state_zenable;
	DWORD state_cullmode;

public:
	MP_Device_WRAP(int width, int height, HWND hwnd);

	LPDIRECT3D9 GetDirect3D(){return direct_3d;}

	LPDIRECT3DDEVICE9 GetDevice3D(){return device_3d;}

public:

	// eng: Creating
	// rus: 
	virtual bool Create();

	// eng: Destroying
	// rus: 
	virtual void Destroy();

	#ifdef MAGIC_3D
	// eng: Setting of scene
	// rus:  
	virtual void SetScene3d(bool camera_mode);

	// eng: Camera position
	// rus:  
	void SetCamera(MP_POSITION& eye, MP_POSITION& at, MP_POSITION& up);
	#endif

	// eng: Beginning of scene drawing
	// rus:   
	virtual void BeginScene();

	// eng: End of scene drawing
	// rus:   
	virtual void EndScene();

	// eng: Indicates that device is lost
	// rus:     
	virtual bool IsLost();

	virtual void SetVertexFormat(MAGIC_RENDERING_START* start);

	virtual void SetRenderState(MAGIC_RENDER_STATE* state);

	virtual void DrawVertices(int starting_index, int indexes_count, int max_vertices);

	virtual MAGIC_ARGB_ENUM GetColorMode(){return MAGIC_ABGR;}

	// eng: Creating of layout
	// rus:   
	virtual MP_LAYOUT* NewLayout();

	// eng: Creating of vertex buffer
	// rus:   
	virtual MP_BUFFER* NewVertexBuffer();

	// eng: Creating of index buffer
	// rus:  
	virtual MP_BUFFER* NewIndexBuffer();

	#ifdef MAGIC_3D
	virtual int GetAdditionalVerticesPosition(){return 0;}
	#else
	virtual int GetAdditionalVerticesPosition(){return 2;}
	virtual int GetAdditionalAttributeLength(int attribute, int index);
	#endif
};

#endif